//==========================================================================
// File : ShlDisp.pas
//
// Definitions for shell functions in Delphi, according to the August 2002
// version of the Microsoft Platform SDK
//
// Author : K. Giddings
//
//
// This file is part of the NSELib library for Delphi.
// NSELib can be found on the web on this URL:
// http://www.whirlingdervishes.com/nselib
// The documentation can be found at this URL:
// http://www.whirlingdervishes.com/nselib/help
//
// This source code is the property of its authors and may not be
// redistributed or reproduced in any form without the written
// permission of the authors.
//
// You may freely use this unit in the creation of any software. 
//
// Copyright (C) 1998-2004 Whirling Dervishes Software
//
// This software is provided 'as-is', without any express or implied
// warranty.  In no event will the authors be held liable for any damages
// arising from the use of this software.
//
//==========================================================================
unit ShlDisp;

interface

uses Windows, ActiveX;
// While most of ShlDisp can be extracted from the shell32 type library,  the
// following interfaces do not appear.

//-------------------------------------------------------------------------
//
// IAutoComplete interface
//
//
// [Member functions]
//
// IAutoComplete::Init(hwndEdit, punkACL, pwszRegKeyPath, pwszQuickComplete)
//   This function initializes an AutoComplete object, telling it
//   what control to subclass, and what list of strings to process.
//
// IAutoComplete::Enable(fEnable)
//   This function enables or disables the AutoComplete functionality.
//
//-------------------------------------------------------------------------
type
  IAutoComplete = interface
    ['{00bb2762-6a77-11d0-a535-00c04fd7d062}']
    function Init(hwndEdit : HWND; punkACL : IUnknown;
                  pwszRegKeyPath : PWideChar;
                  pwszQuickComplete : PWideChar) : HResult; stdcall;
    function Enable(fEnable : bool) : HResult; stdcall;
  end;

const
  //options for IAutoComplete2
  ACO_NONE   = 0;
  ACO_AUTOSUGGEST  = $1;
  ACO_AUTOAPPEND  = $2;
  ACO_SEARCH   = $4;
  ACO_FILTERPREFIXES  = $8;
  ACO_USETAB   = $10;
  ACO_UPDOWNKEYDROPSLIST = $20;
  ACO_RTLREADING  = $40;

type
  IAutoComplete2 = interface(IAutoComplete)
    ['{EAC04BC0-3791-11d2-BB95-0060977B464C}']
    function SetOptions(dwFlag : cardinal) : HResult; stdcall;
    function GetOptions(out pdwFlag : cardinal) : HResult; stdcall;
  end;


// INTERFACE: IEnumACString
//
// This interface was implemented to return autocomplete strings
// into the caller's buffer (to reduce the number of memory allocations).
// A sort index is also returned to control the order of items displayed.
// by autocomplete.  The sort index should be set to zero if unused.
//
// The NextItem method increments the current index by one (similar to Next
// when one item is requested).
//

const
  ACEO_NONE = 0;
	ACEO_MOSTRECENTFIRST	= 1;
	ACEO_FIRSTUNUSED	= $10000;

type
  IEnumACString = interface(IEnumString)
    ['{8E74C210-CF9D-4eaf-A403-7356428F0A5A}']
    function NextItem(out pszUrl : PWideChar; cchMax : cardinal; out pulSortIndex : cardinal) : HResult; stdcall;
    function SetEnumOptions(dwOptions : cardinal) : HResult; stdcall;
    function GetEnumOptions(out pdwOptions : cardinal) : HResult; stdcall;
  end;

// INTERFACE: IAsyncOperation
//
// This interface was implemented to turn some previously synchronous
// interfaces into async.  The following example is for
// doing the IDataObject::Drop() operation asynchronously.
//
// Sometimes the rendering of the IDataObject data (IDataObject::GetData() or
// STGMEDIUM.pStream->Read()) can be time intensive.  The IDropTarget
// may want to do this on another thread.
//
// Implementation Check list:
// DoDragDrop Caller:
//    If this code can support asynch operations, then it needs to
//    QueryInterface() the IDataObject for IAsyncOperation.
//    IAsyncOperation::SetAsyncMode(VARIANT_TRUE).
//    After calling DoDragDrop(), call InOperation().  If any call fails
//    or InOperation() return FALSE, use the pdwEffect returned by DoDragDrop()
//    and the operation completed synchrously.
//
// OleSetClipboard Caller:
//    If this code can support asynch operations, then it needs to
//    QueryInterface() the IDataObject for IAsyncOperation.  Then call
//    IAsyncOperation::SetAsyncMode(VARIANT_TRUE).
//    If any of that fails, the final dwEffect should be passed to the IDataObject via
//    CFSTR_PERFORMEDDROPEFFECT.
//
// IDataObect Object:
//    IAsyncOperation::GetAsyncMode() should return whatever was last passed in
//          fDoOpAsync to ::SetAsyncMode() or VARIANT_FALSE if ::SetAsyncMode()
//          was never called.
//    IAsyncOperation::SetAsyncMode() should AddRef and store paocb.
//    IAsyncOperation::StartOperation() should store the fact that this was called and
//          cause InOperation() to return VARIANT_TRUE.  pbcReserved is not used and needs
//          to be NULL.
//    IAsyncOperation::InOperation() should return VARIANT_TRUE only if ::StartOperation()
//          was called.
//    IAsyncOperation::EndOperation() needs to call paocbpaocb->EndOperation() with the same
//          parameters.  Then release paocb.
//    IDataObject::SetData(CFSTR_PERFORMEDDROPEFFECT) When this happens, call
//          EndOperation(<into VAR>S_OK, NULL, <into VAR>dwEffect) and pass the dwEffect from the hglobal.
//
// IDropTarget Object:
//    IDropTarget::Drop() If asynch operations aren't supported, nothing is required.
//          The asynch operation can only happen if GetAsyncMode() returns VARIANT_TRUE.
//          Before starting the asynch operation, StartOperation(NULL) needs to be called before
//          returning from IDropTarget::Drop().
// -----------------------------------------------------------------------------
// NB although the docs imply IAsyncOperation should be using bool for the
// pfIsOpAsync parameter, explorer chacks for a 1 as a true value instead of a
// -1 as documented.
type
  IAsyncOperation = interface
    ['{3D8B0590-F691-11d2-8EA9-006097DF5BD4}']
    function SetAsyncMode(fDoOpAsync : integer) : HResult; stdcall;
    function GetAsyncMode(out pfIsOpAsync : integer) : HResult; stdcall;
    function StartOperation(pbcReserved : IBindCtx) : HResult; stdcall;
    function InOperation(out pfInAsyncOp : integer) : HResult; stdcall;
    function EndOperation(Res : HResult; pbcReserved : IBindCtx; dwEffects : cardinal) : HResult; stdcall;
  end;

implementation

end.
